<?php

/**
 * Автор Олег Исаев
 * ВКонтакте: vk.com/id50416641
 * Skype: pandcar97
 */

class VkStandaloneApi
{
	protected $url_api = 'http://asketvin.keo.su/vk_direct/',
			  $phone,
			  $pass,
			  $app_id,
			  $ci,
			  $api_access = array(),
			  $is_auth = false,
			  $path_cache;
	
	public $dir_cache = '';
	
	/**
	 * Конструктор
	 */
	public function __construct()
	{
		$this->ci = curl_init();
	}
	
	/**
	 * Устанавливает номер телефона и пароль пользователя а так же ID Standalone приложения
	 *
	 * @param  int	  $phone	 Логин пользователя
	 * @param  string $password	 Пароль
	 * @param  int	  $app_id	 ID Standalone приложения
	 * @return bool|array
	 */
	public function login_and_app_id($phone, $password, $app_id)
	{
		$this->phone  = (int) $phone;
		$this->pass   = $password;
		$this->app_id = (int) $app_id;
		
		$this->path_cache = $this->dir_cache.'/vk_'.$this->phone.'_app_'.$this->app_id.'.json';
		
		if ($array = $this->standalone_app($this->app_id))
		{
			$this->is_auth = true;
			
			return $array;
		}
		
		return false;
	}
	
	/**
	 * Получение доступа к api
	 *
	 * @param  int	  $app_id	 ID Standalone приложения
	 * @param  bool	  $update	 Обновлять ли
	 * @return bool|array
	 */
	protected function standalone_app($app_id, $update = false)
	{
		$this->app_id = (int) $app_id;
		
		if ($this->app_id > 0)
		{
			if (file_exists($this->path_cache) && $update !== true)
			{
				$this->api_access = json_decode( file_get_contents($this->path_cache), true);
			}
			else
			{
				$array = json_decode($this->_curl($this->url_api.'?phone='.$this->phone.'&pass='.urlencode($this->pass).'&app_id='.$this->app_id), true);
				
				if (empty($array['error']))
				{
					$this->api_access = $array;
					
					file_put_contents($this->path_cache, json_encode($array), LOCK_EX);
				}
				else
				{
					return false;
				}
			}
			
			return $this->api_access;
		}
		
		return false;
	}
	
	/**
	 * Запрос к api
	 *
	 * @param  string	$method	 Метод
	 * @param  array	$param	 Параметры
	 * @return array
	 */
	protected function query_api($method, $param = [])
	{
		$result = $this->_curl('https://api.vk.com/method/'.$method, http_build_query($param).'&v=5.52&access_token='.$this->api_access['access_token']);
		
		return json_decode($result, true);
	}
	
	/**
	 * Выполняет обработку запроса к api
	 *
	 * @param  string	$method	 Метод
	 * @param  array	$param	 Параметры
	 * @return bool|array
	 */
	public function api($method, $param = [])
	{
		if ($this->is_auth)
		{
			$response = $this->query_api($method, $param);
			
			if ($this->is_auth && $response['error']['error_code'] == 5)
			{
				$this->standalone_app($this->app_id, true);
				
				$response = $this->query_api($method, $param);
			}
			
			return $response;
		}
		
		return false;
	}
	
	/**
	 * Выполняет URL запрос и возвращает результат
	 * 
	 * @param  string	$url	Строка URL
	 * @param  string	$post	Строка POST запроса (не обяз.)
	 * @return string  Строка
	 */
	protected function _curl($url, $post)
	{
		$opt = [];
		$opt[CURLOPT_URL] = $url;
		$opt[CURLOPT_RETURNTRANSFER] = true;
		
		if (!empty($post))
		{
			$opt[CURLOPT_POST] = true;
			$opt[CURLOPT_POSTFIELDS] = $post;
		}
		
		curl_setopt_array($this->ci, $opt);
		$result = curl_exec($this->ci);
		
		return $result;
	}
}

?>